local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicSmallGate.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Aligned Small Gate materials
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Materials/redRoof", StoneGateSetPrefix .. "_SGATE_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Materials/murChateau", StoneGateSetPrefix .. "_SGATE_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_SGATE_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_SGATE_WHITE_STONE_FLOOR_GROUND_MATERIAL")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Materials/whiteStoneFrame", StoneGateSetPrefix .. "_SGATE_WHITE_STONE_FRAME_MATERIAL")

-- Register Aligned Small Gate prefab nodes
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart", StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_1_SGate_Tiling1Part", StoneGateSetPrefix .. "_SGATE_TILING1_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_1_SGate_Tiling2Part", StoneGateSetPrefix .. "_SGATE_TILING2_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_1_SGate_Tiling3Part", StoneGateSetPrefix .. "_SGATE_TILING3_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootHalfRoofUpperPart", StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_UPPER_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootHalfRoofExtPart", StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_EXT_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopArchPart", StoneGateSetPrefix .. "_SGATE_TOP_ARCH_PREFAB")
StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopUpperPart", StoneGateSetPrefix .. "_SGATE_TOP_UPPER_PREFAB")
--------------------------------------------------------------------------

-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:registerAsset({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }}
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

local defaultNodePrefixList = {
	StoneGateSetPrefix .. "_SGATE_TOP_ARCH",
	StoneGateSetPrefix .. "_SGATE_TOP_UPPER",
	StoneGateSetPrefix .. "_SGATE_TILING1",
	StoneGateSetPrefix .. "_SGATE_TILING2",
	StoneGateSetPrefix .. "_SGATE_TILING3",
}

-- Register simple building part assets
for i, nodePrefix in ipairs(defaultNodePrefixList) do
	registerDefaultBuildingPart(nodePrefix)
end

function registerSmallGate(_option)
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootUpper" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_ROOT_UPPER" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootExt" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_ROOT_EXT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling1Arch" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TILING1_ARCH" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling1Upper" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TILING1_UPPER" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling1Ext" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TILING1_EXT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling2Upper" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_2_SGate_Tiling3Upper" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofUpper" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_UPPER" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofArch" .. _option .. "Part", StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_ARCH" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TILING1_ARCH" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TILING1_UPPER" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TILING1_EXT" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_UPPER" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_ARCH" .. _option)
   
    StoneGate:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootUpper" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 8 }
    })
    StoneGate:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootExt" .. _option .. "Part", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 8 }
    })

    -- Register the stone gate set small gate root part with scaleable arch
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_ARCH" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_ARCH" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_ARCH" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_ARCH_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_ARCH" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_PART"
         	},
		MinimumScale = 1,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register the stone gate set small gate root part with scaleable upper part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_UPPER" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_UPPER" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_UPPER" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_UPPER" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_UPPER_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_UPPER" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = true
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register the stone gate set small gate extension root part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_EXT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_EXT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_EXT" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateExtensionBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_EXT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_UPPER_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_EXT" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = true
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register the stone gate set small gate root part with scaleable arch and half roof
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_ARCH" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_ARCH" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_ARCH" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateHalfRoofBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_ARCH" .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_ARCH" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART",
			StoneGateSetPrefix .. "_SGATE_TILING3_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_PART"
         	},
		MinimumScale = 1,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false,
                        OrientationOffset = -90
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register the stone gate set small gate root part with scaleable upper part and half roof
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_UPPER" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_UPPER" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_UPPER" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateHalfRoofBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_UPPER_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_UPPER" .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_UPPER" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = true,
                        OrientationOffset = -90
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })

    -- Register the stone gate set small gate extension root part with half roof
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_EXT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_EXT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_EXT" .. _option .. "_DESC",
	Category = "CORE",
        BuildingZone = _SmallGateExtensionHalfRoofBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_ROOT_HALF_ROOF_EXT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_SGATE_TOP_HALF_ROOF_UPPER" .. _option .. "_PART",
		FillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING3_UPPER" .. _option .. "_PART",
			StoneGateSetPrefix .. "_SGATE_TILING2_UPPER" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_SGATE_TILING1_EXT" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = true,
                        OrientationOffset = -90

                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	}
    })
end

registerSmallGate("") -- with stone floor
registerSmallGate("P") -- with wooden floor

-- Register grass clearing components

StoneGate:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_0_SGate_RootPart", {
	DataType = "COMP_DIRT_RECTANGLE",
	Size = { 4, 8 }
})


function registerRampartPart(_nameFbx, _nameAsset, _resource, _buildingPartType)
    StoneGate:registerAssetId("models/basic/basicSmallGate.fbx/Prefab/smallGate/" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_DESC",
	Category = "EXTENSION",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_SGATE_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = _resource, Quantity = 2*_NoSandbox }
		}}
	}
    })

    StoneGate:registerPrefabComponent("models/basic/basicSmallGate.fbx/Prefab/smallGate/" .. _nameFbx .. "Part", {
	DataType = "COMP_BUILDING_PART",
	BuildingPartType = _buildingPartType
    })
end

registerRampartPart( "step_4_SGate_RampartA", "RAMPART_A", "STONE", "MAJOR" )
registerRampartPart( "step_4_SGate_RampartClosingL", "RAMPART_CLOSING_L", "STONE", "MAJOR" )
registerRampartPart( "step_4_SGate_RampartClosingR", "RAMPART_CLOSING_R", "STONE", "MAJOR" )
registerRampartPart( "step_5_SGate_RoofA45", "ROOFA45", "PLANK", "MAJOR" )
registerRampartPart( "step_5_SGate_RoofA30", "ROOFA30", "PLANK", "MAJOR" )
registerRampartPart( "step_5_SGate_RoofA60", "ROOFA60", "PLANK", "MAJOR" )

-- platforms
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopUpperPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopArchPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofUpperPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofUpperPPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofArchPart", { "PLATFORM" })
StoneGate:configurePrefabFlagList("models/basic/basicSmallGate.fbx/Prefab/smallGate/step_3_SGate_TopHalfRoofArchPPart", { "PLATFORM" })
