local StoneGate = ...

-- Apply building asset processor
StoneGate:registerAssetProcessor("models/basic/basicOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/simpleRoofs/roofOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/roofs/domeRoofs/domeOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
StoneGate:registerAssetProcessor("models/ramparts/rampartOctogonalTower.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

-- Register Octogonal Tower materials
StoneGate:registerAssetId("models/roofs/simpleRoofs/roofOctogonalTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_OTOWER_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/roofs/domeRoofs/domeOctogonalTower.fbx/Materials/redRoof", StoneGateSetPrefix .. "_OTOWER_DOME_RED_TILE_ROOF_MATERIAL")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/murChateau", StoneGateSetPrefix .. "_OTOWER_STONE_MANOR_WALL_MATERIAL")
StoneGate:registerAssetId("models/ramparts/rampartOctogonalTower.fbx/Materials/blueCrenellation", StoneGateSetPrefix .. "_RAMPART_OTOWER_STONE_MANOR_CRENELLATION_MATERIAL")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/murChateauBasement", StoneGateSetPrefix .. "_OTOWER_STONE_MANOR_BASEMENT_MATERIAL")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Materials/whiteStoneGround", StoneGateSetPrefix .. "_OTOWER_WHITE_STONE_FLOOR_GROUND_MATERIAL")

--------------------------------------------------------------------------
-- Register Octogonal Tower prefab nodes
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_3_Octogonal_TopPart", StoneGateSetPrefix .. "_OTOWER_TOP_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootPart", StoneGateSetPrefix .. "_OTOWER_ROOT_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1Part", StoneGateSetPrefix .. "_OTOWER_TILING1_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2Part", StoneGateSetPrefix .. "_OTOWER_TILING2_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3Part", StoneGateSetPrefix .. "_OTOWER_TILING3_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootPPart", StoneGateSetPrefix .. "_OTOWER_ROOTB_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1PPart", StoneGateSetPrefix .. "_OTOWER_TILING1B_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2PPart", StoneGateSetPrefix .. "_OTOWER_TILING2B_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3PPart", StoneGateSetPrefix .. "_OTOWER_TILING3B_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_RootHALFWHITEPart", StoneGateSetPrefix .. "_OTOWER_ROOTHALFWHITE_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1HALFWHITEPart", StoneGateSetPrefix .. "_OTOWER_TILING1HALFWHITE_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2HALFWHITEPart", StoneGateSetPrefix .. "_OTOWER_TILING2HALFWHITE_PREFAB")
StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3HALFWHITEPart", StoneGateSetPrefix .. "_OTOWER_TILING3HALFWHITE_PREFAB")


-- Create default building part registering function 
function registerDefaultBuildingPart(_nodePrefix)
	StoneGate:registerAsset({
		DataType = "BUILDING_PART",
		Id = _nodePrefix .. "_PART",
		ConstructorData = {
                    DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
                    CoreObjectPrefab = _nodePrefix .. "_PREFAB"
                },
                IsVisibleWhenBuilt = false, -- to mask the green diamond in top, only visible in preview
	        Cost = {
		    UpkeepCost = {
			{ Resource = "GOLD", Quantity = 1*_NoSandbox }
		    },
		    ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		    }}
	        },
	        EstateSplendor = {
		    { Estate = "KINGDOM", Quantity = 0.1 }
	    }
	})
end

registerDefaultBuildingPart(StoneGateSetPrefix .. "_OTOWER_TOP")

function registerOctogonalTower(_option)
    -- Register Octogonal Tower prefab nodes
    StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_Root" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_1_Octogonal_Tiling1" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING1" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling2" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING2" .. _option .. "_PREFAB")
    StoneGate:registerAssetId("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_2_Octogonal_Tiling3" .. _option .. "Part", StoneGateSetPrefix .. "_OTOWER_TILING3" .. _option .. "_PREFAB")
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_OTOWER_TILING1" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_OTOWER_TILING2" .. _option)
    registerDefaultBuildingPart(StoneGateSetPrefix .. "_OTOWER_TILING3" .. _option)

    -- Register the stone gate set main root part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PART",
	Name = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_NAME",
	Description = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_DESC",
	Category = StoneGateSetPrefix .. "_OTOWER_PARTSETLIST_NAME",
	BuildingZone = _OTowerBuildingZone,
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER", -- Since there are different types of building constructor, 'DataType' is necessary here
		CoreObjectPrefab = StoneGateSetPrefix .. "_OTOWER_ROOT" .. _option .. "_PREFAB",
		EndPart = StoneGateSetPrefix .. "_OTOWER_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_OTOWER_TILING2" .. _option .. "_PART",
			StoneGateSetPrefix .. "_OTOWER_TILING3" .. _option .. "_PART"
                },
		BasementFillerList = {
			StoneGateSetPrefix .. "_OTOWER_TILING1" .. _option .. "_PART"
         	},
		MinimumScale = 0,
		IsVertical = true,
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	},
	EstateSplendor = {
		{ Estate = "KINGDOM", Quantity = 1 }
	},
        VillagerCapacity = 8,
--	AssetBuildingFunction = registerAssignableFunction(StoneGateSetPrefix, "OTOWER_ROOT" .. _option)
    })

    -- Register grass clearing components
    StoneGate:registerPrefabComponent("models/basic/basicOctogonalTower.fbx/Prefab/octogonalTower/step_0_Octogonal_Root" .. _option .. "Part", {
	DataType = "COMP_DIRT_CIRCLE",
	HardRadius = 4.6,
        SoftRadius = 4.6,
        IsPermanent = true
    })
end

registerOctogonalTower("") -- with stone ground
registerOctogonalTower("P") -- with wooden ground
registerOctogonalTower("HALFWHITE") -- Half white with stone ground

-- Roof...
 
function registerRoofPart(_nameFbxFile, _nameFbx, _nameAsset, _angle)
    StoneGate:registerAssetId("models/" .. _nameFbxFile .. "/Prefab/octogonalTower/" .. _nameFbx .. _angle .. "Part", StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PREFAB")

-- Register the stone gate set decoration roof part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PART",
	Name = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_NAME",
	Description = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_DESC",
	Category = StoneGateSetPrefix .. "_OTOWER_PARTSETLIST_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_" .. _nameAsset .. _angle .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "PLANK", Quantity = 5*_NoSandbox }
		}}
	}
    })
end

registerRoofPart( "roofs/simpleRoofs/roofOctogonalTower.fbx", "OctogonalRoof", "OTOWER_ROOF", "45" )
registerRoofPart( "roofs/simpleRoofs/roofOctogonalTower.fbx", "OctogonalRoof", "OTOWER_ROOF", "30" )
registerRoofPart( "roofs/simpleRoofs/roofOctogonalTower.fbx", "OctogonalRoof", "OTOWER_ROOF", "60" )
registerRoofPart( "roofs/domeRoofs/domeOctogonalTower.fbx", "OctogonalRoofDome", "OTOWER_ROOF_DOME", "" )

---- Ramparts...

function registerRampartPart(_nameFbx, _nameAsset)
    StoneGate:registerAssetId("models/ramparts/rampartOctogonalTower.fbx/Prefab/octogonalTower/step_4_Octogonal_" .. _nameFbx .. "Part", StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PREFAB")

-- Register the stone gate set decoration rampart part
    StoneGate:registerAsset({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PART",
	Name = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_NAME",
	Description = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_DESC",
	Category = StoneGateSetPrefix .. "_OTOWER_PARTSETLIST_NAME",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_OTOWER_" .. _nameAsset .. "_PREFAB",
                MiniatureConfig = {
                        DataType = "BUILDING_MINIATURE_CONFIG",
                        ClipUnderGround = false
                }
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		ResourceNeededList = {{ -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "STONE", Quantity = 2*_NoSandbox }
		}}
	}
    })
end

registerRampartPart( "Rampart", "RAMPART" )
registerRampartPart( "Rampart1Side", "RAMPART_1SIDE" )
registerRampartPart( "Rampart1SideOpened", "RAMPART_1SIDE_OPENED" )
